#!/usr/bin/env python
# -*- coding: utf-8 -*-

import subprocess
from enum import Enum
from .pmkid import AttackPMKID
from .wep import AttackWEP
from .wpa import AttackWPA
from .wps import AttackWPS
from ..config import Configuration
from ..model.target import WPSState
from ..util.color import Color

class Answer(Enum):
    Skip = 1
    ExitOrReturn = 2
    Continue = 3
    Ignore = 4

class AttackAll(object):
    @classmethod
    def attack_multiple(cls, targets):
        """
        Attacks all given `targets` (list[wifite.model.target]) until user interruption.
        Returns: Number of targets that were attacked (int)
        """
        # Safety check: ensure targets is not None
        if targets is None:
            Color.pl('{!} {R}Error: No targets provided for attack{W}')
            return 0

        if any(t.wps for t in targets) and not AttackWPS.can_attack_wps():
            # Warn that WPS attacks are not available.
            Color.pl('{!} {O}Note: WPS attacks are not possible because you do not have {C}reaver{O} nor {C}bully{W}')

        attacked_targets = 0
        targets_remaining = len(targets)
        for index, target in enumerate(targets, start=1):
            if Configuration.attack_max != 0 and index > Configuration.attack_max:
                print(("Attacked %d targets, stopping because of the --first flag" % Configuration.attack_max))
                break
            attacked_targets += 1
            targets_remaining -= 1

            # Periodic cleanup to prevent file descriptor leaks
            if index % 5 == 0:  # Every 5 targets
                from ..util.process import Process
                Process.check_fd_limit()

            bssid = target.bssid
            essid = target.essid if target.essid_known else '{O}ESSID unknown{W}'

            Color.pl('\n{+} ({G}%d{W}/{G}%d{W})'
                     % (index, len(targets)) + ' Starting attacks against {C}%s{W} ({C}%s{W})' % (bssid, essid))

            should_continue = cls.attack_single(target, targets_remaining)
            if not should_continue:
                break

        return attacked_targets

    @classmethod
    def attack_single(cls, target, targets_remaining):
        """
        Attacks a single `target` (wifite.model.target).
        Returns: True if attacks should continue, False otherwise.
        """
        global attack
        if 'MGT' in target.authentication:
            Color.pl("\n{!}{O}Skipping. Target is using {C}WPA-Enterprise {O}and can not be cracked.")
            return True

        attacks = []

        if Configuration.use_eviltwin:
            # TODO: EvilTwin attack
            pass

        elif target.primary_encryption == 'WEP':
            attacks.append(AttackWEP(target))

        elif target.primary_encryption.startswith('WPA'): # Covers WPA, WPA2, WPA3
            # WPA can have multiple attack vectors:

            # WPS
            # For WPA3, WPS is not applicable in the same way.
            # WPS is generally being phased out with WPA3, though some transition modes might exist.
            # We will only attempt WPS if it's explicitly WPA or WPA2 (not WPA3).
            if target.primary_encryption != 'WPA3' and \
               not Configuration.use_pmkid_only and \
               target.wps is WPSState.UNLOCKED and \
               AttackWPS.can_attack_wps():

                # Pixie-Dust
                if Configuration.wps_pixie:
                    attacks.append(AttackWPS(target, pixie_dust=True))

                # Null PIN zero-day attack
                if Configuration.wps_pin: # This implies not wps_pixie_only
                    attacks.append(AttackWPS(target, pixie_dust=False, null_pin=True))

                # PIN attack
                if Configuration.wps_pin: # This implies not wps_pixie_only
                    attacks.append(AttackWPS(target, pixie_dust=False))

            # PMKID and Handshake attacks are applicable to WPA, WPA2, and WPA3
            if not Configuration.wps_only: # If --wps-only is not set
                # PMKID
                if not Configuration.dont_use_pmkid: # If --no-pmkid is not set
                    attacks.append(AttackPMKID(target))

                # Handshake capture
                if not Configuration.use_pmkid_only: # If --pmkid (means pmkid-only) is not set
                    attacks.append(AttackWPA(target))
            elif target.primary_encryption == 'WPA3' and Configuration.wps_only:
                # Special case: If it's WPA3 and --wps-only is specified,
                # WPS attacks are skipped. We should still allow PMKID/Handshake for WPA3.
                Color.pl('{!} {O}Note: --wps-only is active, but target is WPA3. WPS attacks are not applicable.')
                Color.pl('{+} {C}Proceeding with PMKID and Handshake attacks for WPA3 target.{W}')
                if not Configuration.dont_use_pmkid:
                    attacks.append(AttackPMKID(target))
                if not Configuration.use_pmkid_only:
                    attacks.append(AttackWPA(target))


        if not attacks:
            Color.pl('{!} {R}Error: {O}Unable to attack: no attacks available')
            return True  # Keep attacking other targets (skip)

        while attacks:
            # Needed by infinite attack mode in order to count how many targets were attacked
            target.attacked = True
            attack = attacks.pop(0)
            try:
                result = attack.run()
                if result:
                    break  # Attack was successful, stop other attacks.
            except (OSError, IOError) as e:
                # File system or process errors
                Color.pl('\r {!} {R}System Error{W}: %s' % str(e))
                continue
            except subprocess.CalledProcessError as e:
                # Command execution failures
                Color.pl('\r {!} {R}Command Failed{W}: %s' % str(e))
                continue
            except ValueError as e:
                # Invalid data or configuration
                Color.pl('\r {!} {R}Configuration Error{W}: %s' % str(e))
                continue
            except PermissionError as e:
                # Permission issues
                Color.pl('\r {!} {R}Permission Error{W}: %s' % str(e))
                continue
            except Exception as e:
                # Unexpected errors - still catch but log more info
                Color.pl('\r {!} {R}Unexpected Error{W}: %s' % str(e))
                if Configuration.verbose > 0:
                    Color.pexception(e)
                # Force cleanup on unexpected errors to prevent resource leaks
                from ..util.process import ProcessManager, Process
                ProcessManager().cleanup_all()
                Process.cleanup_zombies()
                continue
            except KeyboardInterrupt:
                Color.pl('\n{!} {O}Interrupted{W}\n')
                answer = cls.user_wants_to_continue(targets_remaining, len(attacks))
                if answer == Answer.Continue:
                    continue  # Keep attacking the same target (continue)
                elif answer == Answer.Skip:
                    return True  # Keep attacking other targets (skip)
                elif answer == Answer.Ignore:
                    from ..model.result import CrackResult
                    CrackResult.ignore_target(target)
                    return True  # Ignore current target and keep attacking other targets (ignore)
                else:
                    return False  # Stop all attacks (exit)

        if attack.success:
            attack.crack_result.save()

        return True  # Keep attacking other targets

    @classmethod
    def user_wants_to_continue(cls, targets_remaining, attacks_remaining=0):
        """
        Asks user if attacks should continue onto other targets
        Returns:
            Answer.Skip if the user wants to skip the current target
            Answer.Ignore if the user wants to ignore the current target
            Answer.Continue if the user wants to continue to the next attack on the current target
            Answer.ExitOrReturn if the user wants to stop the remaining attacks
        """
        if attacks_remaining == 0 and targets_remaining == 0:
            return  # No targets or attacksleft, drop out

        prompt_list = []
        if attacks_remaining > 0:
            prompt_list.append(Color.s('{C}%d{W} attack(s)' % attacks_remaining))
        if targets_remaining > 0:
            prompt_list.append(Color.s('{C}%d{W} target(s)' % targets_remaining))
        prompt = ' and '.join(prompt_list) + ' remain'
        Color.pl('{+} %s' % prompt)

        prompt = '{+} Do you want to'
        options = '('

        if attacks_remaining > 0:
            prompt += ' {G}continue{W} attacking,'
            options += '{G}c{W}{D}, {W}'

        if targets_remaining > 0:
            prompt += ' {O}skip{W} to the next target,'
            options += '{O}s{W}{D}, {W}'

        prompt += ' skip and {P}ignore{W} current target,'
        options += '{P}i{W}{D}, {W}'

        if Configuration.infinite_mode:
            options += '{R}r{W})'
            prompt += ' or {R}return{W} to scanning %s? {C}' % options
        else:
            options += '{R}e{W})'
            prompt += ' or {R}exit{W} %s? {C}' % options

        Color.p(prompt)
        try:
            answer = input().lower()
        except KeyboardInterrupt:
            # If user presses Ctrl+C during input, default to exit
            Color.pl('\n{!} {O}Interrupted during input, exiting...{W}')
            return Answer.ExitOrReturn

        if answer.startswith('s'):
            return Answer.Skip
        elif answer.startswith('e') or answer.startswith('r'):
            return Answer.ExitOrReturn  # Exit/Return
        elif answer.startswith('i'):
            return Answer.Ignore  # Ignore
        else:
            return Answer.Continue  # Continue



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import contextlib
import time
import signal
import os
import atexit
import threading
import subprocess
from subprocess import Popen, PIPE
from ..util.color import Color
from ..config import Configuration


class ProcessManager:
    """Global process manager to track and cleanup all processes"""
    _instance = None
    _lock = threading.Lock()

    def __new__(cls):
        if cls._instance is None:
            with cls._lock:
                if cls._instance is None:
                    cls._instance = super().__new__(cls)
                    cls._instance._processes = set()
                    cls._instance._registered_cleanup = False
        return cls._instance

    def register_process(self, process):
        """Register a process for cleanup tracking"""
        with self._lock:
            # Limit total tracked processes to prevent resource exhaustion
            if len(self._processes) > 50:  # Reasonable limit
                # Clean up finished processes first
                finished_processes = {p for p in self._processes if hasattr(p, 'is_running') and not p.is_running()}
                self._processes -= finished_processes

                # If still too many, force cleanup of oldest
                if len(self._processes) > 50:
                    oldest_processes = list(self._processes)[:10]  # Remove oldest 10
                    for p in oldest_processes:
                        try:
                            p.force_kill()
                        except:
                            pass
                    self._processes -= set(oldest_processes)

            self._processes.add(process)
            if not self._registered_cleanup:
                atexit.register(self.cleanup_all)
                self._registered_cleanup = True

    def unregister_process(self, process):
        """Unregister a process (when it's properly cleaned up)"""
        with self._lock:
            self._processes.discard(process)

    def cleanup_all(self):
        """Emergency cleanup of all tracked processes"""
        with self._lock:
            for process in list(self._processes):
                try:
                    if process.pid and process.pid.poll() is None:
                        process.force_kill()
                except:
                    pass  # Ignore errors during emergency cleanup
            self._processes.clear()


class Process(object):
    """ Represents a running/ran process with enhanced cleanup """

    @staticmethod
    def devnull():
        """ Helper method for opening devnull """
        return open('/dev/null', 'w')

    @staticmethod
    def call(command, cwd=None, shell=False):
        """
            Calls a command (either string or list of args).
            Returns tuple:
                (stdout, stderr)
        """
        if type(command) is not str or ' ' in command or shell:
            shell = True
            if Configuration.verbose > 1:
                Color.pe('\n {C}[?] {W} Executing (Shell): {B}%s{W}' % command)
        else:
            shell = False
            if Configuration.verbose > 1:
                Color.pe('\n {C}[?]{W} Executing: {B}%s{W}' % command)

        pid = Popen(command, cwd=cwd, stdout=PIPE, stderr=PIPE, shell=shell)
        pid.wait()
        (stdout, stderr) = pid.communicate()

        # Python 3 compatibility
        if type(stdout) is bytes:
            stdout = stdout.decode('utf-8')
        if type(stderr) is bytes:
            stderr = stderr.decode('utf-8')

        if Configuration.verbose > 1 and stdout is not None and stdout.strip() != '':
            Color.pe('{P} [stdout] %s{W}' % '\n [stdout] '.join(stdout.strip().split('\n')))
        if Configuration.verbose > 1 and stderr is not None and stderr.strip() != '':
            Color.pe('{P} [stderr] %s{W}' % '\n [stderr] '.join(stderr.strip().split('\n')))

        return stdout, stderr

    @staticmethod
    def exists(program):
        """ Checks if program is installed on this system """

        if Configuration.initialized and program in set(Configuration.existing_commands.keys()):
            return Configuration.existing_commands[program]

        p2 = Process(['which', program])
        stdout = p2.stdout().strip()
        stderr = p2.stderr().strip()

        exist = not stdout == stderr == ''
        if Configuration.initialized:
            Configuration.existing_commands.update({program: exist})
        return exist

    def __init__(self, command, devnull=False, stdout=PIPE, stderr=PIPE, cwd=None, bufsize=0, stdin=PIPE):
        """ Starts executing command """

        if type(command) is str:
            # Commands have to be a list
            command = command.split(' ')

        self.command = command
        self._cleaned_up = False
        self._manager = ProcessManager()

        if Configuration.verbose > 1:
            Color.pe('\n {C}[?] {W} Executing: {B}%s{W}' % ' '.join(command))

        self.out = None
        self.err = None
        if devnull:
            sout = Process.devnull()
            serr = Process.devnull()
        else:
            sout = stdout
            serr = stderr

        self.start_time = time.time()

        # Add file descriptor limit checking
        try:
            self.pid = Popen(command, stdout=sout, stderr=serr, stdin=stdin, cwd=cwd, bufsize=bufsize)
        except OSError as e:
            if e.errno == 24:  # Too many open files
                # Force cleanup of zombie processes and try again
                ProcessManager().cleanup_all()
                Process.cleanup_zombies()
                time.sleep(0.1)  # Brief pause
                try:
                    self.pid = Popen(command, stdout=sout, stderr=serr, stdin=stdin, cwd=cwd, bufsize=bufsize)
                except OSError as e2:
                    if e2.errno == 24:
                        raise OSError("Too many open files - system resource exhausted. Try reducing concurrent attacks.") from e2
                    raise
            else:
                raise

        # Register with process manager for cleanup tracking
        self._manager.register_process(self)

    def __enter__(self):
        """Context manager entry"""
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """Context manager exit - ensures cleanup"""
        self.cleanup()

    def __del__(self):
        """
            Ran when object is GC'd.
            If process is still running at this point, it should die.
        """
        self.cleanup()

    def stdout(self):
        """ Waits for process to finish, returns stdout output """
        self.get_output()
        if Configuration.verbose > 1 and self.out is not None and self.out.strip() != '':
            Color.pe('{P} [stdout] %s{W}' % '\n [stdout] '.join(self.out.strip().split('\n')))
        return self.out

    def stderr(self):
        """ Waits for process to finish, returns stderr output """
        self.get_output()
        if Configuration.verbose > 1 and self.err is not None and self.err.strip() != '':
            Color.pe('{P} [stderr] %s{W}' % '\n [stderr] '.join(self.err.strip().split('\n')))
        return self.err

    def stdoutln(self):
        return self.pid.stdout.readline()

    def stderrln(self):
        return self.pid.stderr.readline()

    def stdin(self, text):
        if self.pid.stdin:
            self.pid.stdin.write(text.encode('utf-8'))
            self.pid.stdin.flush()

    def get_output(self):
        """ Waits for process to finish, sets stdout & stderr """
        if self.pid.poll() is None:
            try:
                self.pid.wait(timeout=30)  # 30 second timeout
            except subprocess.TimeoutExpired:
                # Force kill if process doesn't finish in time
                self.force_kill()
        if self.out is None:
            try:
                (self.out, self.err) = self.pid.communicate(timeout=10)  # 10 second timeout
            except subprocess.TimeoutExpired:
                # Force kill and get partial output
                self.force_kill()
                (self.out, self.err) = self.pid.communicate()

        if type(self.out) is bytes:
            self.out = self.out.decode('utf-8')

        if type(self.err) is bytes:
            self.err = self.err.decode('utf-8')

        # Limit output size to prevent memory bloat
        max_output_size = 1024 * 1024  # 1MB limit
        if self.out and len(self.out) > max_output_size:
            self.out = self.out[-max_output_size:] + "\n[... output truncated ...]"
        if self.err and len(self.err) > max_output_size:
            self.err = self.err[-max_output_size:] + "\n[... error output truncated ...]"

        return self.out, self.err

    def poll(self):
        """ Returns exit code if process is dead, otherwise 'None' """
        return self.pid.poll()

    def wait(self):
        self.pid.wait()

    def running_time(self):
        """ Returns number of seconds since process was started """
        return int(time.time() - self.start_time)

    def cleanup(self):
        """Properly cleanup the process"""
        if getattr(self, '_cleaned_up', False):
            return

        try:
            if hasattr(self, 'pid') and self.pid and self.pid.poll() is None:
                self.interrupt()
        except:
            pass  # Ignore errors during cleanup
        finally:
            self._cleaned_up = True
            # Use try-except for manager cleanup to handle shutdown scenarios
            try:
                if hasattr(self, '_manager'):
                    self._manager.unregister_process(self)
            except:
                pass  # Ignore errors during shutdown

    def interrupt(self, wait_time=2.0):
        """
            Send interrupt to current process.
            If process fails to exit within `wait_time` seconds, terminates it.
        """
        if not hasattr(self, 'pid') or not self.pid:
            return

        try:
            self._graceful_shutdown(wait_time)
        except OSError as e:
            if 'No such process' in str(e):
                return
            # Process might be in zombie state, try to clean it up
            try:
                self.pid.wait()
            except:
                pass

    def _graceful_shutdown(self, wait_time):
        """Attempt graceful shutdown with escalating signals"""
        if self.pid.poll() is not None:
            return  # Already dead

        pid = self.pid.pid
        cmd = ' '.join(self.command) if isinstance(self.command, list) else str(self.command)

        if Configuration.verbose > 1:
            Color.pe('\n {C}[?] {W} sending interrupt to PID %d (%s)' % (pid, cmd))

        # Step 1: Try SIGINT (graceful)
        try:
            os.kill(pid, signal.SIGINT)
        except OSError:
            return  # Process already dead

        # Wait for graceful shutdown
        start_time = time.time()
        while self.pid.poll() is None and (time.time() - start_time) < wait_time:
            try:
                time.sleep(0.1)
            except KeyboardInterrupt:
                break  # User wants to abort

        # Step 2: Try SIGTERM if still running
        if self.pid.poll() is None:
            if Configuration.verbose > 1:
                Color.pe('\n {C}[?] {W} Process still running, sending SIGTERM to PID %d' % pid)
            try:
                os.kill(pid, signal.SIGTERM)
                time.sleep(1.0)  # Give it a moment
            except OSError:
                pass

        # Step 3: Force kill with SIGKILL if still running
        if self.pid.poll() is None:
            if Configuration.verbose > 1:
                Color.pe('\n {C}[?] {W} Force killing PID %d' % pid)
            try:
                os.kill(pid, signal.SIGKILL)
                self.pid.kill()
            except OSError:
                pass

        # Final cleanup
        try:
            self.pid.wait()
        except:
            pass

    def force_kill(self):
        """Force kill the process immediately"""
        if not hasattr(self, 'pid') or not self.pid:
            return

        try:
            if self.pid.poll() is None:
                self.pid.kill()
                self.pid.wait()
        except:
            pass

    def is_running(self):
        """Check if process is still running"""
        return hasattr(self, 'pid') and self.pid and self.pid.poll() is None

    @staticmethod
    def cleanup_zombies():
        """Clean up any zombie processes"""
        try:
            # Reap any zombie children
            while True:
                try:
                    pid, status = os.waitpid(-1, os.WNOHANG)
                    if pid == 0:
                        break
                    if Configuration.verbose > 2:
                        Color.pe(f'\n {{C}}[?]{{W}} Reaped zombie process PID {pid}')
                except OSError:
                    break
        except:
            pass

    @staticmethod
    def get_open_fd_count():
        """Get current number of open file descriptors"""
        try:
            import os
            proc_fd_dir = f'/proc/{os.getpid()}/fd'
            if os.path.exists(proc_fd_dir):
                return len(os.listdir(proc_fd_dir))
        except:
            pass
        return -1

    @staticmethod
    def check_fd_limit():
        """Check if we're approaching file descriptor limits"""
        try:
            import resource
            soft_limit, hard_limit = resource.getrlimit(resource.RLIMIT_NOFILE)
            current_fds = Process.get_open_fd_count()

            if current_fds > 0 and current_fds > (soft_limit * 0.8):  # 80% of limit
                Color.pl(f'\n{{!}} {{O}}Warning: High file descriptor usage ({current_fds}/{soft_limit}){{W}}')
                ProcessManager().cleanup_all()
                Process.cleanup_zombies()
                return True
        except:
            pass
        return False


if __name__ == '__main__':
    Configuration.initialize(False)
    p = Process('ls')
    print((p.stdout()))
    print((p.stderr()))
    p.interrupt()

    # Calling as list of arguments
    (out, err) = Process.call(['ls', '-lah'])
    print(out)
    print(err)

    print('\n---------------------\n')

    # Calling as string
    (out, err) = Process.call('ls -l | head -2')
    print(out)
    print(err)

    print(f""""reaver" exists: {Process.exists('reaver')}""")

    # Test on never-ending process
    p = Process('yes')
    print('Running yes...')
    time.sleep(1)
    print('yes should stop now')
    # After program loses reference to instance in 'p', process dies.



#!/usr/bin/env python3
import os
import sys
import csv
import requests
from datetime import datetime
from typing import List

# keep the same import style as your example (adjust to your project layout)
from ..config import Configuration
from ..util.color import Color
from ..util.process import Process

class DBUpdater:
    """Updates a local database of MAC address prefixes to vendor names from IEEE registries.
    """

    # Registry URLs (same idea as your original script)
    SOURCES = {
        "OUI":   "https://standards-oui.ieee.org/oui/oui.csv",
        "MAM":   "https://standards-oui.ieee.org/oui28/mam.csv",
        "OUI36": "https://standards-oui.ieee.org/oui36/oui36.csv",
        "IAB":   "https://standards-oui.ieee.org/iab/iab.csv",
    }

    DEFAULT_FILENAME = "ieee-oui.txt"

    @classmethod
    def run(cls):
        Configuration.initialize(False)

        filename = Configuration.db_filename
        verbose = bool(Configuration.verbose)

        if os.path.exists(filename):
            up_to_date, last_updated = cls.is_up_to_date(filename)
            if up_to_date:
                Color.pl('{+} {G}Database is up to date ({C}%s{G}). Last update date: {C}%s{W}' % (filename, last_updated))
                return
            if verbose:
                Color.pl('{!} {O}Deleting existing {R}%s{W}' % filename)
            os.remove(filename)

        try:
            total_written = cls.update_all(filename, verbose=verbose)
        except KeyboardInterrupt:
            Color.pl('\n{!} {O}Interrupted by user{W}')
            return

        Color.pl('\n\n{+} {G}Done{W} - Total entries written: {C}%d{W}' % total_written)

    @ classmethod
    def update_all(cls, filename: str, verbose: bool = False) -> int:
        """Loop selected sources, fetch, parse and append to filename. Returns count written."""
        written_total = 0
        with open(filename, "w", encoding="utf-8") as outfile:
            date_str = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
            outfile.write(f"# Registry Vendor List\n# Generated {date_str}\n")
            for key in cls.SOURCES.keys():
                url = cls.SOURCES.get(key)
                Color.pl('\n{+} Processing {C}%s{W} from {O}%s{W}' % (key, url))
                try:
                    csv_content = cls.fetch_csv(url, verbose=verbose)
                    written = cls.parse_and_write_csv(csv_content, outfile, key, verbose=verbose)
                    written_total += written
                except Exception as e:
                    print(f"Error processing {key}: {e}", file=sys.stderr)
        return written_total

    @classmethod
    def fetch_csv(cls, url: str, verbose: bool = False) -> str:
        """Download CSV content (boilerplate; uses requests)."""
        headers = {"User-Agent": "Mozilla/5.0 (compatible; FetchOUI/1.0; +https://github.com/kimocoder/wifite2)"}
        if verbose:
            Color.pl('  →  Fetching %s' % url)
        response = requests.get(url, headers=headers, timeout=30)
        if not response.ok:
            raise RuntimeError(f"Failed to fetch {url}: {response.status_code} {response.reason}")
        if len(response.content) == 0:
            raise RuntimeError(f"Empty response from {url}")
        return response.text

    @classmethod
    def parse_and_write_csv(cls, csv_content: str, outfile, key: str, verbose: bool = False) -> int:
        """Parse CSV content and write MAC\tVendor lines to outfile (boilerplate)."""
        reader = csv.DictReader(csv_content.splitlines())
        outfile.write(f"\n#\n# Start of {key} registry data\n#\n")
        count = 0
        for row in reader:
            mac = row.get("Assignment") or row.get("Registry") or ""
            vendor = row.get("Organization Name") or row.get("Organization") or ""
            vendor = (vendor or "").strip()
            if mac and vendor:
                outfile.write(f"{mac}\t{vendor}\n")
                count += 1
        outfile.write(f"#\n# End of {key} registry data. {count} entries.\n#\n")

        Color.p('     Wrote {C}%d{W} entries from source: {C}%s{W}' % (count, key))
        return count

    def is_up_to_date(filename: str) -> bool:
        mtime = os.path.getmtime(filename)
        last_update = datetime.fromtimestamp(mtime).strftime("%Y-%m-%d %H:%M:%S")
        age_seconds = datetime.now().timestamp() - mtime
        return age_seconds < (7 * 24 * 3600), last_update #if file is older than 7 days it is not up to date


if __name__ == '__main__':
    DBUpdater.run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import re
import signal
import subprocess
import time

from .dependency import Dependency
from .ip import Ip
from .iw import Iw
from ..config import Configuration
from ..util.color import Color
from ..util.process import Process


class AirmonIface:
    def __init__(self, phy, interface, driver, chipset):
        self.phy = phy
        self.interface = interface
        self.driver = driver
        self.chipset = chipset

    # Max length of fields.
    # Used for printing a table of interfaces.
    INTERFACE_LEN = 12
    PHY_LEN = 6
    DRIVER_LEN = 20
    CHIPSET_LEN = 30

    def __str__(self):
        """ Colored string representation of interface """
        s = ''
        s += Color.s(f'{self.interface.ljust(self.INTERFACE_LEN)}')
        s += Color.s(f'{self.phy.ljust(self.PHY_LEN)}')
        s += Color.s(f'{self.driver.ljust(self.DRIVER_LEN)}')
        s += Color.s(f'{self.chipset.ljust(self.CHIPSET_LEN)}')
        return s

    @staticmethod
    def menu_header():
        """ Colored header row for interfaces """
        s = '    ' + 'Interface'.ljust(AirmonIface.INTERFACE_LEN)
        s += 'PHY'.ljust(AirmonIface.PHY_LEN)
        s += 'Driver'.ljust(AirmonIface.DRIVER_LEN)
        s += 'Chipset'.ljust(AirmonIface.CHIPSET_LEN)
        s += '\n'
        s += '-' * \
             (AirmonIface.INTERFACE_LEN + AirmonIface.PHY_LEN + AirmonIface.DRIVER_LEN + AirmonIface.CHIPSET_LEN + 3)
        return s


class Airmon(Dependency):
    """ Wrapper around the 'airmon-ng' program """
    dependency_required = True
    dependency_name = 'airmon-ng'
    dependency_url = 'https://www.aircrack-ng.org/install.html'
    chipset_table = 'https://wikidevi.com/wiki/Wireless_adapters/Chipset_table'
    base_interface = None
    killed_network_manager = False
    use_ipiw = False
    isdeprecated = False

    # Drivers that need to be manually put into monitor mode
    BAD_DRIVERS = ['rtl8821au']
    DEPRECATED_DRIVERS = ['rtl8723cs']
    # see if_arp.h
    ARPHRD_ETHER = 1  # managed
    ARPHRD_IEEE80211_RADIOTAP = 803  # monitor

    def __init__(self):
        self.interfaces = None
        self.refresh()

    def refresh(self):
        """ Get airmon-recognized interfaces """
        self.interfaces = Airmon.get_interfaces()

    def print_menu(self):
        """ Prints menu """
        print((AirmonIface.menu_header()))
        for idx, interface in enumerate(self.interfaces, start=1):
            Color.pl(' {G}%d{W}. %s' % (idx, interface))

    def get(self, index):
        """ Gets interface at index (starts at 1) """
        if type(index) is str:
            index = int(index)
        return self.interfaces[index - 1]

    @staticmethod
    def get_interfaces():
        """Returns List of AirmonIface objects known by airmon-ng"""
        interfaces = []
        p2 = Process('airmon-ng')
        for line in p2.stdout().split('\n'):
            # [PHY ]IFACE DRIVER CHIPSET
            airmon_re = re.compile(r'^(?:([^\t]*)\t+)?([^\t]*)\t+([^\t]*)\t+([^\t]*)$')
            matches = airmon_re.match(line)
            if not matches:
                continue

            phy, interface, driver, chipset = matches.groups()
            if phy in ['PHY', 'Interface']:
                continue

            if len(interface.strip()) == 0:
                continue

            interfaces.append(AirmonIface(phy, interface, driver, chipset))

        return interfaces

    @staticmethod
    def get_iface_info(interface_name):
        """
        Get interface info (driver, chipset), based on interface name.
        Returns an AirmonIface if interface name is found by airmon-ng or None
        """
        return next((intf_adapter for intf_adapter in Airmon.get_interfaces() if intf_adapter.interface == interface_name), None)

    @staticmethod
    def start_bad_driver(interface, isdeprecated=False):
        """
        Manually put interface into monitor mode (no airmon-ng or vif).
        Fix for bad drivers like the rtl8812AU.
        """
        Ip.down(interface)
        if isdeprecated:
            Process(['iwconfig', interface, 'mode', 'monitor']).stdout()
        else:
            Iw.mode(interface, 'monitor')
        Ip.up(interface)

        # /sys/class/net/wlan0/type
        iface_type_path = os.path.join('/sys/class/net', interface, 'type')
        if os.path.exists(iface_type_path):
            with open(iface_type_path, 'r'):
                pass

        return interface

    @staticmethod
    def stop_bad_driver(interface):
        """
        Manually put interface into managed mode (no airmon-ng or vif).
        Fix for bad drivers like the rtl8812AU.
        """
        # Get driver info for ICNSS2 check
        iface_info = Airmon.get_iface_info(interface)
        if iface_info and iface_info.driver == 'icnss2':
            Color.p('{+} ICNSS2 driver detected for %s. Running "svc wifi disable"... ' % interface)
            # Run 'svc wifi disable'
            proc = Process(['svc', 'wifi', 'disable'])
            # proc.wait() # Wait for command to complete.
            # Alternatively, use Process.call for simpler cases if output isn't critical and we just need to run it
            # For now, let's assume we want to wait and check for errors, similar to other Process calls.
            stdout, stderr = proc.communicate() # communicate calls wait() internally
            if proc.poll() == 0:
                Color.pl('{G}success!{W}')
            else:
                Color.pl('{R}failed.{W}')
                if stdout: Color.pl('{O}STDOUT: %s{W}' % stdout.strip())
                if stderr: Color.pl('{R}STDERR: %s{W}' % stderr.strip())
                # Decide if we should proceed or raise an error. For now, let's proceed.

        Ip.down(interface)
        Iw.mode(interface, 'managed')
        Ip.up(interface)

        # /sys/class/net/wlan0/type
        iface_type_path = os.path.join('/sys/class/net', interface, 'type')
        if os.path.exists(iface_type_path):
            with open(iface_type_path, 'r'):
                pass

        return interface

    @classmethod
    def start(cls, interface):
        """
            Starts an interface (iface) in monitor mode
            Args:
                interface: Interface to start
            Returns:
                enabled_interface: Enabled interface
            Throws:
                Exception: If no interface is found
        """
        # Get interface name from input
        if type(interface) == AirmonIface:
            iface_name = interface.interface
            driver = interface.driver
        else:
            iface_name = interface
            driver = None # We'll try to fetch this if needed

        # Remember this as the 'base' interface.
        Airmon.base_interface = iface_name

        # Try ICNSS2-specific activation first
        if iface_name == 'wlan0':
            # Try to get driver info if not already available
            if driver is None:
                iface_obj = Airmon.get_iface_info(iface_name)
                if iface_obj:
                    driver = iface_obj.driver

            if driver == 'icnss2':
                Color.p('{+} Attempting {G}ICNSS2 monitor mode{W} on {C}%s{W}... ' % iface_name)
                con_mode_path = '/sys/module/wlan/parameters/con_mode'
                if os.path.exists(con_mode_path):
                    try:
                        # Ensure interface is down before changing mode
                        Ip.down(iface_name)
                        subprocess.run(['echo', '4', '>', con_mode_path], shell=True, check=True, capture_output=True)
                        # Bring interface up
                        Ip.up(iface_name)
                        # Verify it's in monitor mode
                        if Iw.is_monitor(iface_name):
                            Color.pl('{G}enabled (ICNSS2 specific)!{W}')
                            # TODO: Consider if we need to set cls.use_ipiw or other flags here
                            return iface_name
                        else:
                            Color.pl('{O}failed (ICNSS2 specific, could not verify monitor mode). Trying other methods...{W}')
                            # Attempt to revert if possible, or let subsequent methods handle it
                            # Process(['echo', '0', '>', con_mode_path], shell=True) # Optional: revert
                    except subprocess.CalledProcessError as e:
                        Color.pl('{R}failed (ICNSS2 specific command error: %s). Trying other methods...{W}' % e.stderr.decode().strip())
                    except (OSError, IOError) as e:
                        Color.pl('{R}failed (ICNSS2 I/O error: %s). Trying other methods...{W}' % str(e))
                    except ValueError as e:
                        Color.pl('{R}failed (ICNSS2 config error: %s). Trying other methods...{W}' % str(e))
                    except Exception as e:
                        Color.pl('{R}failed (ICNSS2 unexpected error: %s). Trying other methods...{W}' % str(e))
                else:
                    Color.pl('{O}con_mode path not found for ICNSS2. Trying other methods...{W}')


        # If driver is deprecated then skip airmon-ng
        if driver not in Airmon.DEPRECATED_DRIVERS:
            # Try to enable using Airmon-ng first (for better compatibility)
            Color.p('{+} Enabling {G}monitor mode{W} on {C}%s{W}... ' % iface_name)
            airmon_output = Process(['airmon-ng', 'start', iface_name]).stdout()
            enabled_interface = Airmon._parse_airmon_start(airmon_output)
            
            # Debug output for troubleshooting airmon-ng parsing issues
            if Configuration.verbose > 0:
                print(f"\nDEBUG: Full airmon_output = {repr(airmon_output)}")
                print(f"DEBUG: enabled_interface = {repr(enabled_interface)}")

                # Debug the parsing step by step
                lines = airmon_output.split('\n')
                for i, line in enumerate(lines):
                    if 'mac80211 monitor mode' in line:
                        print(f"DEBUG: Found monitor mode line {i}: {repr(line)}")
        else:
            enabled_interface = None

        # if it fails, try to use ip/iw
        if enabled_interface is None:
            Airmon.isdeprecated = driver in Airmon.DEPRECATED_DRIVERS
            enabled_interface = Airmon.start_bad_driver(iface_name, Airmon.isdeprecated)
        else:
            # If not, just set for us know how it went in monitor mode
            cls.use_ipiw = True

        if not Airmon.isdeprecated:
            # if that also fails, just give up
            if enabled_interface is None:
                Color.pl('{R}failed to enable monitor mode using standard methods.{W}')
                raise Exception('Failed to enable monitor mode')

            # Assert that there is an interface in monitor mode
            # interfaces = Iw.get_interfaces(mode='monitor') # This might be too early if mon iface has a new name
            # We rely on Iw.is_monitor(enabled_interface) or similar check later.
            if not Iw.is_monitor(enabled_interface):
                 # Airmon-ng sometimes creates a new interface (e.g. wlan0mon)
                 # We need to check if *any* monitor interface was created if enabled_interface itself is not in mon mode.
                 # However, our _parse_airmon_start should return the *new* monitor interface name.
                Color.pl('{R}interface %s not in monitor mode after airmon-ng/iw.{W}' % enabled_interface)
                raise Exception(f'Interface {enabled_interface} not in monitor mode after airmon-ng/iw')


        # No errors found; the device 'enabled_iface' was put into Mode:Monitor.
        Color.pl('{G}enabled{W}!')

        return enabled_interface

    @staticmethod
    def _parse_airmon_start(airmon_output):
        """Find the interface put into monitor mode (if any)"""
        # airmon-ng output examples:
        # (mac80211 monitor mode vif enabled for [phy10]wlan0 on [phy10]wlan0mon)
        # (mac80211 monitor mode already enabled for [phy0]wlxd037456283c3 on [phy0]10)
        # (mac80211 monitor mode vif enabled for [phy5]wlxd037456283c3 on [phy5]wlxd037456283c3mon)

        # Try to match from the "on" part first - this is the actual monitor interface
        # Updated regex to handle interface names that may contain numbers at the start
        enabled_on_re = re.compile(r'.*\(mac80211 monitor mode (?:(?:vif )?enabled|already enabled) (?:for [^ ]+ )?on (?:\[\w+])?([a-zA-Z][\w-]+(?:mon)?)\)?.*')

        # Fallback: try to match from the "for" part if "on" part is just numbers (channel)
        enabled_for_re = re.compile(r'.*\(mac80211 monitor mode (?:(?:vif )?enabled|already enabled) for (?:\[\w+])?(\w+).*on (?:\[\w+])?\d+\)?.*')
        lines = airmon_output.split('\n')

        for index, line in enumerate(lines):
            if 'mac80211 monitor mode' not in line:
                continue
            
            if Configuration.verbose > 0:
                print(f"DEBUG: Parsing line: {repr(line)}")
            
            # First try to get interface from "on" part if it looks like an interface name
            if matches := enabled_on_re.match(line):
                result = matches.group(1)
                if Configuration.verbose > 0:
                    print(f"DEBUG: enabled_on_re matched: {repr(result)}")
                return result
            # Fallback to "for" part if "on" part is just a channel number
            elif matches := enabled_for_re.match(line):
                result = matches.group(1)
                if Configuration.verbose > 0:
                    print(f"DEBUG: enabled_for_re matched: {repr(result)}")
                return result
            # Legacy fallback
            elif "monitor mode enabled" in line:
                result = line.split()[-1]
                if Configuration.verbose > 0:
                    print(f"DEBUG: legacy fallback matched: {repr(result)}")
                return result
            else:
                if Configuration.verbose > 0:
                    print(f"DEBUG: No regex matched this line")

        return None

    @classmethod
    def stop(cls, interface):
        Color.p('{!}{W} Disabling {O}monitor{W} mode on {R}%s{W}...\n' % interface)

        if cls.use_ipiw:
            enabled_interface = disabled_interface = Airmon.stop_bad_driver(interface)
        else:
            airmon_output = Process(['airmon-ng', 'stop', interface]).stdout()
            disabled_interface, enabled_interface = Airmon._parse_airmon_stop(airmon_output)

        if disabled_interface:
            Color.pl('{+}{W} Disabled monitor mode on {G}%s{W}' % disabled_interface)
        else:
            Color.pl('{!} {O}Could not disable {R}%s{W}' % interface)

        return disabled_interface, enabled_interface

    @staticmethod
    def _parse_airmon_stop(airmon_output):
        """Find the interface taken out of into monitor mode (if any)"""
        # airmon-ng 1.2rc2 output: (mac80211 monitor mode vif enabled for [phy10]wlan0 on [phy10]wlan0mon)
        disabled_re = re.compile(r'\s*\(mac80211 monitor mode (?:vif )?disabled for (?:\[\w+])?(\w+)\)\s*')

        # airmon-ng 1.2rc1 output: wlan0mon (removed)
        removed_re = re.compile(r'([a-zA-Z\d]+).*\(removed\)')

        # Enabled interface: (mac80211 station mode vif enabled on [phy4]wlan0)
        enabled_re = re.compile(r'\s*\(mac80211 station mode (?:vif )?enabled on (?:\[\w+])?(\w+)\)\s*')

        disabled_interface = None
        enabled_interface = None
        for line in airmon_output.split('\n'):
            if matches := disabled_re.match(line):
                disabled_interface = matches.group(1)

            if matches := removed_re.match(line):
                disabled_interface = matches.group(1)

            if matches := enabled_re.match(line):
                enabled_interface = matches.group(1)

        return disabled_interface, enabled_interface

    @staticmethod
    def ask():
        """
        Asks user to define which wireless interface to use.
        Does not ask if:
            1. There is already an interface in monitor mode, or
            2. There is only one wireless interface (automatically selected).
        Puts selected device into Monitor Mode.
        """
        Airmon.terminate_conflicting_processes()

        Color.p('\n{+} Looking for {C}wireless interfaces{W}...')
        monitor_interfaces = Iw.get_interfaces(mode='monitor')
        if len(monitor_interfaces) == 1:
            # Assume we're using the device already in monitor mode
            interface = monitor_interfaces[0]
            Color.clear_entire_line()
            Color.pl('{+} Using {G}%s{W} already in monitor mode' % interface)
            Airmon.base_interface = None
            return interface

        Color.clear_entire_line()
        Color.p('{+} Checking {C}airmon-ng{W}...')

        a = Airmon()
        if len(a.interfaces) == 0:
            # No interfaces found
            Color.pl('\n{!} {O}airmon-ng did not find {R}any{O} wireless interfaces')
            Color.pl('{!} {O}Make sure your wireless device is connected')
            Color.pl('{!} {O}See {C}https://www.aircrack-ng.org/doku.php?id=airmon-ng{O} for more info{W}')
            raise Exception('airmon-ng did not find any wireless interfaces')

        Color.clear_entire_line()
        a.print_menu()

        Color.pl('')

        if len(a.interfaces) == 1:
            # Only one interface, assume this is the one to use
            choice = 1
        else:
            # Multiple interfaces found
            Color.p('{+} Select wireless interface ({G}1-%d{W}): ' % len(a.interfaces))
            choice = input()

        selected = a.get(choice)

        if a.get(choice).interface in monitor_interfaces:
            Color.pl('{+} {G}%s{W} is already in monitor mode' % selected.interface)
        else:
            selected.interface = Airmon.start(selected)

        return selected.interface

    @staticmethod
    def terminate_conflicting_processes():
        """ Deletes conflicting processes reported by airmon-ng """
        airmon_output = Process(['airmon-ng', 'check']).stdout()

        # Checking for systemd, otherwise assume openrc

        if os.path.exists('/usr/lib/systemd/systemd'):
            init_system = 'systemd'
        else:
            init_system = 'openrc'
        # TODO: add support for other unorthodox init systems (maybe?)

        # Conflicting process IDs and names
        pid_pnames = []

        # 2272    dhclient
        # 2293    NetworkManager
        pid_pname_re = re.compile(r'^\s*(\d+)\s*([a-zA-Z\d_\-]+)\s*$')
        for line in airmon_output.split('\n'):
            if match := pid_pname_re.match(line):
                pid_pnames.append((match.group(1), match.group(2)))

        if not pid_pnames:
            return

        if not Configuration.kill_conflicting_processes:
            # Don't kill processes, warn user
            names_and_pids = ', '.join([f'{pname} ({pid})' for pid, pname in pid_pnames])
            Color.pl('{!} {O}Conflicting processes: %s' % names_and_pids)
            Color.pl('{!} {O}If you have problems: {R}kill -9 PID{O} or re-run wifite with {R}--kill{O}{W}')
            return

        Color.pl('{!} {O}Killing {R}%d {O}conflicting processes' % len(pid_pnames))
        for pid, pname in pid_pnames:
            if pname == 'NetworkManager' and Process.exists('systemctl'):
                Process(['systemctl', 'stop', 'NetworkManager']).stdout()
            elif pname == 'network-manager' and Process.exists('service'):
                Process(['service', 'network-manager', 'stop']).stdout()
            elif pname == 'avahi-daemon' and Process.exists('service'):
                Process(['service', 'avahi-daemon', 'stop']).stdout()
            else:
                os.kill(int(pid), signal.SIGKILL)

    @staticmethod
    def put_interface_up(interface):
        Color.p('{!}{W} Putting interface {R}%s{W} {G}up{W}...\n' % interface)
        Ip.up(interface)
        Color.pl('{+}{W} Done !')

    @staticmethod
    def start_network_manager():
        Color.p('{!} {O}start {R}NetworkManager{O}...')

        if Process.exists('service'):
            cmd = 'service networkmanager start'
            proc = Process(cmd)
            (out, err) = proc.get_output()
            if proc.poll() != 0:
                Color.pl(' {R}Error executing {O}%s{W}' % cmd)
                if out is not None and out.strip() != '':
                    Color.pl('{!} {O}STDOUT> %s{W}' % out)
                if err is not None and err.strip() != '':
                    Color.pl('{!} {O}STDERR> %s{W}' % err)
            else:
                Color.pl(' {G}Done{W} ({C}%s{W})' % cmd)
                return

        if Process.exists('systemctl'):
            cmd = 'systemctl start NetworkManager'
            proc = Process(cmd)
            (out, err) = proc.get_output()
            if proc.poll() != 0:
                Color.pl(' {R}Error executing {O}%s{W}' % cmd)
                if out is not None and out.strip() != '':
                    Color.pl('{!} {O}STDOUT> %s{W}' % out)
                if err is not None and err.strip() != '':
                    Color.pl('{!} {O}STDERR> %s{W}' % err)
            else:
                Color.pl(' {G}done{W} ({C}%s{W})' % cmd)
                return
        else:
            Color.pl(' {R}Cannot start NetworkManager: {O}systemctl{R} or {O}service{R} not found{W}')


if __name__ == '__main__':
    stdout = '''
Found 2 processes that could cause trouble.
If airodump-ng, aireplay-ng or airtun-ng stops working after
a short period of time, you may want to run 'airmon-ng check kill'

  PID Name
 5563 avahi-daemon
 5564 avahi-daemon

PHY	Interface	Driver		Chipset

phy0	wlx00c0ca4ecae0	rtl8187		Realtek Semiconductor Corp. RTL8187
Interface 15mon is too long for linux so it will be renamed to the old style (wlan#) name.

                (mac80211 monitor mode vif enabled on [phy0]wlan0mon
                (mac80211 station mode vif disabled for [phy0]wlx00c0ca4ecae0)
    '''
    start_iface = Airmon._parse_airmon_start(stdout)
    print(('start_iface from stdout:', start_iface))

    Configuration.initialize(False)
    iface = Airmon.ask()
    (disabled_iface, enabled_iface) = Airmon.stop(iface)
    print(('Disabled:', disabled_iface))
    print(('Enabled:', enabled_iface))

    print(f""""reaver" exists: {Process.exists('reaver')}""")

    # Test on never-ending process
    p = Process('yes')
    print('Running yes...')
    time.sleep(1)
    print('yes should stop now')
    p.interrupt()



#!/usr/bin/env python3
import argparse
import csv
import requests
import sys
import os
from datetime import datetime

# IEEE registry URLs
IEEE_REG_URLS = {
    "OUI":   "https://standards-oui.ieee.org/oui/oui.csv",
    "MAM":   "https://standards-oui.ieee.org/oui28/mam.csv",
    "OUI36": "https://standards-oui.ieee.org/oui36/oui36.csv",
    "IAB":   "https://standards-oui.ieee.org/iab/iab.csv",
}

DEFAULT_FILENAME = "ieee-oui.txt"


def fetch_csv(url, verbose=False):
    """Download CSV content from a URL."""
    headers = {"User-Agent": "Mozilla/5.0 (compatible; FetchOUI/1.0; +https://github.com/kimocoder/wifite2)"}
    if verbose:
        print(f"→ Fetching {url}")
    response = requests.get(url, headers=headers, timeout=30)
    if not response.ok:
        raise RuntimeError(f"Failed to fetch {url}: {response.status_code} {response.reason}")
    if len(response.content) == 0:
        raise RuntimeError(f"Empty response from {url}")
    if verbose:
        print(f"  Downloaded {len(response.content)} bytes")
    return response.text


def parse_and_write_csv(csv_content, outfile, key, verbose=False):
    """Parse CSV content and write MAC/Vendor to file."""
    reader = csv.DictReader(csv_content.splitlines())
    outfile.write(f"\n#\n# Start of IEEE {key} registry data\n#\n")
    count = 0

    for row in reader:
        # Columns differ slightly between registries, so we handle gracefully
        mac = row.get("Assignment") or row.get("Registry") or ""
        vendor = row.get("Organization Name") or row.get("Organization") or ""
        vendor = vendor.strip()
        if mac and vendor:
            outfile.write(f"{mac}\t{vendor}\n")
            count += 1

    outfile.write(f"#\n# End of IEEE {key} registry data. {count} entries.\n#\n")
    if verbose:
        print(f"  Wrote {count} entries for {key}")
    return count


def main():
    parser = argparse.ArgumentParser(
        description="Fetch the IEEE OUI (manufacturer) registries and write MAC/vendor mappings to a text file."
    )
    parser.add_argument("-f", metavar="FILE", default=DEFAULT_FILENAME,
                        help=f"Output filename (default: {DEFAULT_FILENAME})")
    parser.add_argument("-v", action="store_true", help="Verbose output")
    args = parser.parse_args()

    filename = args.f
    verbose = args.v

    # Delete old file if exists
    if os.path.exists(filename):
        if verbose:
            print(f"Deleting existing {filename}")
        os.remove(filename)

    # Open new output file
    if verbose:
        print(f"Opening {filename} for output")

    with open(filename, "w", encoding="utf-8") as outfile:
        date_str = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        outfile.write(f"# IEEE OUI Vendor List\n# Generated {date_str}\n")

        total_entries = 0
        for key, url in sorted(IEEE_REG_URLS.items()):
            if verbose:
                print(f"\nProcessing IEEE {key} registry data from {url}")
            try:
                content = fetch_csv(url, verbose)
                total_entries += parse_and_write_csv(content, outfile, key, verbose)
            except Exception as e:
                print(f"Error processing {key}: {e}", file=sys.stderr)

    print(f"\nTotal of {total_entries} MAC/Vendor mappings written to {filename}")


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

try:
    from .config import Configuration
except (ValueError, ImportError) as e:
    raise Exception("You may need to run wifite from the root directory (which includes README.md)", e) from e


from .util.color import Color

import os
import subprocess


class Wifite(object):

    def __init__(self):
        """
        Initializes Wifite.
        Checks that its running under *nix, with root permissions and ensures dependencies are installed.
        """

        self.print_banner()

        Configuration.initialize(load_interface=False)

        if os.name == 'nt':
            Color.pl('{!} {R}error: {O}wifite{R} must be run under a {O}*NIX{W}{R} like OS')
            Configuration.exit_gracefully()
        if os.getuid() != 0:
            Color.pl('{!} {R}error: {O}wifite{R} must be run as {O}root{W}')
            Color.pl('{!} {R}re-run with {O}sudo{W}')
            Configuration.exit_gracefully()

        from .tools.dependency import Dependency
        Dependency.run_dependency_check()

    def start(self):
        """
        Starts target-scan + attack loop, or launches utilities depending on user input.
        """
        from .model.result import CrackResult
        from .model.handshake import Handshake
        from .util.crack import CrackHelper
        from .util.dbupdater import DBUpdater

        if Configuration.show_cracked:
            CrackResult.display('cracked')

        elif Configuration.show_ignored:
            CrackResult.display('ignored')

        elif Configuration.check_handshake:
            Handshake.check()

        elif Configuration.crack_handshake:
            CrackHelper.run()

        elif Configuration.update_db:
            DBUpdater.run()

        else:
            Configuration.get_monitor_mode_interface()
            self.scan_and_attack()

    @staticmethod
    def print_banner():
        """Displays ASCII art of the highest caliber."""
        Color.pl(r' {G}  .     {GR}{D}     {W}{G}     .    {W}')
        Color.pl(r' {G}.´  ·  .{GR}{D}     {W}{G}.  ·  `.  {G}wifite2 {D}%s{W}' % Configuration.version)
        Color.pl(r' {G}:  :  : {GR}{D} (¯) {W}{G} :  :  :  {W}{D}a wireless auditor by derv82{W}')
        Color.pl(r' {G}`.  ·  `{GR}{D} /¯\ {W}{G}´  ·  .´  {W}{D}maintained by kimocoder{W}')
        Color.pl(r' {G}  `     {GR}{D}/¯¯¯\{W}{G}     ´    {C}{D}https://github.com/kimocoder/wifite2{W}')
        Color.pl('')

    @staticmethod
    def scan_and_attack():
        """
        1) Scans for targets, asks user to select targets
        2) Attacks each target
        """
        from .util.scanner import Scanner
        from .attack.all import AttackAll

        Color.pl('')

        # Scan (no signal handler during scanning to allow proper target selection)
        s = Scanner()
        do_continue = s.find_targets()
        targets = s.select_targets()

        # Attack modules handle KeyboardInterrupt properly, no global handler needed

        if Configuration.infinite_mode:
            while do_continue:
                AttackAll.attack_multiple(targets)
                do_continue = s.update_targets()
                if not do_continue:
                    break
                targets = s.select_targets()
            attacked_targets = s.get_num_attacked()
        else:
            # Attack
            attacked_targets = AttackAll.attack_multiple(targets)

        Color.pl('{+} Finished attacking {C}%d{W} target(s), exiting' % attacked_targets)




def force_exit_handler(signum, frame):
    """Force exit on multiple Ctrl+C during cleanup"""
    import sys
    print('\n[!] Force exiting...')
    sys.exit(1)

def main():
    try:
        wifite = Wifite()
        wifite.start()
    except (OSError, IOError) as e:
        Color.pl('\n{!} {R}System Error{W}: %s' % str(e))
        Color.pl('\n{!} {R}Exiting{W}\n')
    except subprocess.CalledProcessError as e:
        Color.pl('\n{!} {R}Command Failed{W}: %s' % str(e))
        Color.pl('\n{!} {R}Exiting{W}\n')
    except PermissionError as e:
        Color.pl('\n{!} {R}Permission Error{W}: %s' % str(e))
        Color.pl('\n{!} {R}Try running with sudo{W}\n')
    except KeyboardInterrupt:
        Color.pl('\n{!} {O}Interrupted, Shutting down...{W}')
        # Set up force exit handler for cleanup phase
        import signal
        signal.signal(signal.SIGINT, force_exit_handler)
    except Exception as e:
        Color.pl('\n{!} {R}Unexpected Error{W}: %s' % str(e))
        Color.pexception(e)
        Color.pl('\n{!} {R}Exiting{W}\n')

    finally:
        # Set up aggressive force exit handler during cleanup
        import signal
        import sys

        def emergency_exit(signum, frame):
            print('\n[!] Emergency exit!')
            # Disable atexit callbacks and suppress stderr to prevent ugly exception messages
            import atexit
            import os
            atexit._clear()
            # Redirect stderr to devnull to hide any remaining cleanup exceptions
            os.dup2(os.open(os.devnull, os.O_WRONLY), 2)
            sys.exit(1)

        signal.signal(signal.SIGINT, emergency_exit)

        # Quick cleanup with short timeouts
        try:
            from .util.process import ProcessManager
            import threading

            # Run cleanup in thread with timeout
            cleanup_thread = threading.Thread(target=ProcessManager().cleanup_all)
            cleanup_thread.daemon = True
            cleanup_thread.start()
            cleanup_thread.join(timeout=3)  # 3 second timeout
        except:
            pass  # Ignore cleanup errors

        # Delete Reaver .pcap quickly
        try:
            subprocess.run(["rm", "-f", "reaver_output.pcap"], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL, timeout=2)
        except:
            pass

        # Try graceful exit with timeout
        try:
            import threading

            def graceful_exit():
                Configuration.exit_gracefully()

            exit_thread = threading.Thread(target=graceful_exit)
            exit_thread.daemon = True
            exit_thread.start()
            exit_thread.join(timeout=2)  # 2 second timeout
        except:
            pass

        # Force exit regardless
        sys.exit(0)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .dependency import Dependency
from ..config import Configuration
from ..util.process import Process
from ..util.color import Color
import os

hccapx_autoremove = False  # change this to True if you want the hccapx files to be automatically removed


class Hashcat(Dependency):
    dependency_required = False
    dependency_name = 'hashcat'
    dependency_url = 'https://hashcat.net/hashcat/'

    @staticmethod
    def should_use_force():
        command = ['hashcat', '-I']
        stderr = Process(command).stderr()
        return 'No devices found/left' or 'Unstable OpenCL driver detected!' in stderr

    @staticmethod
    def crack_handshake(handshake_obj, target_is_wpa3_sae, show_command=False):
        """
        Cracks a handshake.
        handshake_obj: A Handshake object (should have .capfile attribute)
        target_is_wpa3_sae: Boolean indicating if the target uses WPA3-SAE
        """
        hash_file = HcxPcapngTool.generate_hash_file(handshake_obj, target_is_wpa3_sae, show_command=show_command)

        key = None
        hashcat_mode = '22001' if target_is_wpa3_sae else '22000'  # Use 22000 instead of deprecated 2500
        file_type_msg = "WPA3-SAE hash" if target_is_wpa3_sae else "WPA/WPA2 hash"

        Color.pl(f"{{+}} {{C}}Attempting to crack {file_type_msg} using Hashcat mode {hashcat_mode}{{W}}")

        # Crack hash_file
        for additional_arg in ([], ['--show']):
            command = [
                'hashcat',
                '--quiet',
                '-m', hashcat_mode,
                hash_file,
                Configuration.wordlist
            ]
            if Hashcat.should_use_force():
                command.append('--force')
            command.extend(additional_arg)
            if show_command:
                Color.pl(f'{{+}} {{D}}Running: {{W}}{{P}}{" ".join(command)}{{W}}')
            process = Process(command)
            stdout, stderr = process.get_output()

            # Check for errors first
            if 'No hashes loaded' in stdout or 'No hashes loaded' in stderr:
                continue  # No valid hashes to crack

            if ':' not in stdout:
                continue  # No cracked results

            # Parse the key from hashcat output
            # Expected format: hash:password
            lines = stdout.strip().split('\n')
            for line in lines:
                if ':' in line and not line.startswith('The plugin') and 'hashcat.net' not in line:
                    # Take the last part after the last colon as the password
                    parts = line.split(':')
                    if len(parts) >= 2:
                        key = parts[-1].strip()
                        if key and len(key) > 0:
                            break
            else:
                continue
            break

        return key

    @staticmethod
    def crack_pmkid(pmkid_file, verbose=False):
        """
        Cracks a given pmkid_file using the PMKID/WPA2 attack (-m 22000)
        Returns:
            Key (str) if found; `None` if not found.
        """

        # Run hashcat once normally, then with --show if it failed
        # To catch cases where the password is already in the pot file.
        for additional_arg in ([], ['--show']):
            command = [
                'hashcat',
                '--quiet',      # Only output the password if found.
                '-m', '22000',  # WPA-PMKID-PBKDF2
                '-a', '0',      # Wordlist attack-mode
                pmkid_file,
                Configuration.wordlist,
                '-w', '3'
            ]
            if Hashcat.should_use_force():
                command.append('--force')
            command.extend(additional_arg)
            if verbose and additional_arg == []:
                Color.pl(f'{{+}} {{D}}Running: {{W}}{{P}}{" ".join(command)}{{W}}')

            # TODO: Check status of hashcat (%); it's impossible with --quiet

            hashcat_proc = Process(command)
            hashcat_proc.wait()
            stdout = hashcat_proc.stdout()

            if ':' not in stdout:
                # Failed
                continue
            else:
                # Hashcat PMKID output format: hash*bssid*station*essid:password
                # We only want the password (last part after the last colon)
                return stdout.strip().split(':')[-1]


class HcxDumpTool(Dependency):
    dependency_required = False
    dependency_name = 'hcxdumptool'
    dependency_url = 'apt install hcxdumptool'

    def __init__(self, target, pcapng_file):
        if os.path.exists(pcapng_file):
            os.remove(pcapng_file)

        command = [
            'hcxdumptool',
            '-i', Configuration.interface,
            '-c', str(target.channel) + 'a',
            '-w', pcapng_file
        ]

        self.proc = Process(command)

    def poll(self):
        return self.proc.poll()

    def interrupt(self):
        if hasattr(self, 'proc') and self.proc:
            self.proc.interrupt()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.interrupt()


class HcxPcapngTool(Dependency):
    dependency_required = False
    dependency_name = 'hcxpcapngtool'
    dependency_url = 'apt install hcxtools'

    def __init__(self, target):
        self.target = target
        self.bssid = self.target.bssid.lower().replace(':', '')
        self.pmkid_file = Configuration.temp(f'pmkid-{self.bssid}.22000')

    @staticmethod
    def generate_hash_file(handshake_obj, is_wpa3_sae, show_command=False):
        """
        Generates a hash file suitable for Hashcat.
        For WPA/WPA2, generates .hccapx (for mode 2500).
        For WPA3-SAE, generates a text hash file (for mode 22001).
        """
        if is_wpa3_sae:
            hash_file = Configuration.temp('generated.sae.22001')
            hcx_args = ['--sae-hccapx', hash_file] # hcxpcapngtool uses --sae-hccapx to output WPA3 SAE hashes
        else:
            hash_file = Configuration.temp('generated.22000')
            hcx_args = ['-o', hash_file]  # For mode 22000, still use -o but different file extension

        if os.path.exists(hash_file):
            os.remove(hash_file)

        command = [
            'hcxpcapngtool',
            *hcx_args,
            handshake_obj.capfile # Assuming handshake_obj has a capfile attribute
        ]

        if show_command:
            Color.pl('{+} {D}Running: {W}{P}%s{W}' % ' '.join(command))

        process = Process(command)
        stdout, stderr = process.get_output()
        if not os.path.exists(hash_file) or os.path.getsize(hash_file) == 0:
            error_msg = f'Failed to generate {"SAE hash" if is_wpa3_sae else ".hccapx"} file.'
            error_msg += f'\nOutput from hcxpcapngtool:\nSTDOUT: {stdout}\nSTDERR: {stderr}'
            # Also include tshark check for WPA3
            if is_wpa3_sae:
                from .tshark import Tshark
                tshark_check_cmd = ['tshark', '-r', handshake_obj.capfile, '-Y', 'eapol && wlan.rsn.akms.type == 0.0.9f.6'] # OUI 00-0F-AC, type 8 or 9 for SAE
                # Type 8 (SAE) and Type 9 (SAE FT)
                # Alternative check might be wlan.rsn.ie.akms.selector == 0x000fac08 for SAE
                tshark_process = Process(tshark_check_cmd)
                tshark_stdout, _ = tshark_process.get_output()
                if not tshark_stdout:
                    error_msg += '\nAdditionally, tshark found no SAE AKM in the capture file. Ensure it is a valid WPA3-SAE handshake.'
                else:
                    error_msg += '\nTshark output for SAE AKM check:\n' + tshark_stdout

            raise ValueError(error_msg)
        return hash_file

    @staticmethod
    def generate_john_file(handshake, show_command=False):
        john_file = Configuration.temp('generated.john')
        if os.path.exists(john_file):
            os.remove(john_file)

        command = [
            'hcxpcapngtool',
            '--john', john_file,
            handshake.capfile
        ]

        if show_command:
            Color.pl('{+} {D}Running: {W}{P}%s{W}' % ' '.join(command))

        process = Process(command)
        stdout, stderr = process.get_output()
        if not os.path.exists(john_file):
            raise ValueError('Failed to generate .john file, output: \n%s\n%s' % (
                stdout, stderr))

        return john_file

    def get_pmkid_hash(self, pcapng_file):
        if os.path.exists(self.pmkid_file):
            os.remove(self.pmkid_file)

        command = 'hcxpcapngtool -o ' + self.pmkid_file + " " + pcapng_file
        hcxpcap_proc = Process(command)
        hcxpcap_proc.wait()

        if not os.path.exists(self.pmkid_file):
            return None

        with open(self.pmkid_file, 'r') as f:
            output = f.read()
            # Each line looks like:
            # hash*bssid*station*essid

        # Note: The dumptool will record *anything* it finds, ignoring the filterlist.
        # Check that we got the right target (filter by BSSID)
        matching_pmkid_hash = None
        for line in output.split('\n'):
            fields = line.split('*')
            if len(fields) >= 3 and fields[3].lower() == self.bssid:
                # Found it
                matching_pmkid_hash = line
                break

        os.remove(self.pmkid_file)
        return matching_pmkid_hash



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from _pyrepl.readline import raw_input

# Fix for raw_input on python3: https://stackoverflow.com/a/7321970
try:
    input = raw_input
except NameError:
    pass


#!/usr/bin/env python3

import sys
sys.path.insert(0, '.')

from wifite.tools.airmon import Airmon

# Test with the exact current output
test_output = """
Found 6 processes that could cause trouble.
Kill them using 'airmon-ng check kill' before putting
the card in monitor mode, they will interfere by changing channels
and sometimes putting the interface back in managed mode

    PID Name
   7044 avahi-daemon
   7105 avahi-daemon
   7183 wpa_supplicant
  25765 NetworkManager
2503235 dhclient
2523490 dhclient

PHY     Interface       Driver          Chipset

phy1    wlp4s0          iwlwifi         Intel Corporation Wi-Fi 6E(802.11ax) AX210/AX1675* 2x2 [Typhoon Peak] (rev 1a)
phy0    wlxd037456283c3 rtl8xxxu        TP-Link TL-WN821N v5/v6 [RTL8192EU]
                (mac80211 monitor mode already enabled for [phy0]wlxd037456283c3 on [phy0]10)
"""

print("Testing exact airmon-ng output...")
result = Airmon._parse_airmon_start(test_output)
print(f"Result: '{result}'")

# Let's debug the regex matching
import re

enabled_on_re = re.compile(r'.*\(mac80211 monitor mode (?:(?:vif )?enabled|already enabled) (?:for [^ ]+ )?on (?:\[\w+])?([a-zA-Z]\w+)\)?.*')
enabled_for_re = re.compile(r'.*\(mac80211 monitor mode (?:(?:vif )?enabled|already enabled) for (?:\[\w+])?(\w+).*on (?:\[\w+])?\d+\)?.*')

lines = test_output.split('\n')
for i, line in enumerate(lines):
    if 'mac80211 monitor mode' in line:
        print(f"\nLine {i}: {repr(line)}")

        match1 = enabled_on_re.match(line)
        if match1:
            print(f"  enabled_on_re matched: '{match1.group(1)}'")
        else:
            print("  enabled_on_re: no match")

        match2 = enabled_for_re.match(line)
        if match2:
            print(f"  enabled_for_re matched: '{match2.group(1)}'")
        else:
            print("  enabled_for_re: no match")

        # Check the legacy fallback
        if "monitor mode enabled" in line:
            print(f"  legacy fallback would return: '{line.split()[-1]}'")
        else:
            print("  legacy fallback: no match")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import gc
from ..util.color import Color
from ..config import Configuration


class MemoryMonitor:
    """Memory monitoring and optimization utilities"""

    @staticmethod
    def get_memory_usage():
        """Get current memory usage in MB"""
        try:
            import psutil
            process = psutil.Process(os.getpid())
            return process.memory_info().rss / 1024 / 1024  # Convert to MB
        except ImportError:
            # Fallback method using /proc/self/status on Linux
            try:
                with open('/proc/self/status', 'r') as f:
                    for line in f:
                        if line.startswith('VmRSS:'):
                            # Extract memory in kB and convert to MB
                            mem_kb = int(line.split()[1])
                            return mem_kb / 1024
            except (OSError, IOError, ValueError):
                pass
        return None

    @staticmethod
    def check_memory_usage(threshold_mb=500):
        """Check if memory usage exceeds threshold and warn user"""
        memory_mb = MemoryMonitor.get_memory_usage()
        if memory_mb and memory_mb > threshold_mb:
            if Configuration.verbose > 0:
                Color.pl('{!} {O}Memory usage: {R}%.1f MB{W} (threshold: %d MB)' % (memory_mb, threshold_mb))
            return True
        return False

    @staticmethod
    def force_cleanup():
        """Force garbage collection and memory cleanup"""
        # Clear any large temporary data structures
        gc.collect()

        # Clean up configuration cache
        Configuration.cleanup_memory()

        if Configuration.verbose > 1:
            memory_after = MemoryMonitor.get_memory_usage()
            if memory_after:
                Color.pl('{+} {G}Memory cleanup completed{W}: %.1f MB' % memory_after)

    @staticmethod
    def periodic_check(counter, check_interval=100):
        """Perform periodic memory checks and cleanup"""
        if counter % check_interval == 0:
            if MemoryMonitor.check_memory_usage():
                MemoryMonitor.force_cleanup()
                return True
        return False
